/******************************************************************************
 * menuman.cpp
 * Dropdown menu manager.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----             FILE OPEN AND CLOSE MANAGEMENT               ----!!!!
 ******************************************************************************/
const TCHAR szFileNone[] = _T("None");

typedef enum FileType_e {
	eFILETYPE_SMP = 0,
	eFILETYPE_INS,
	eFILETYPE_MUS,
	eFILETYPE_SFX,
	eFILETYPE_ALL,

	eFILETYPE_FOOTER
} FileType_t;

const TCHAR szFileFilter_SMP[] = _T("Sample Pack (*.smp)\0*.SMP\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_INS[] = _T("Instrument Pack (*.ins)\0*.INS\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_MUS[] = _T("Music Track (*.mus)\0*.MUS\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_SFX[] = _T("Sound Effect (*.sfx)\0*.SFX\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_WAV[] = _T("Sounds (*.wav)\0*.WAV\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_ALL[] = _T("All Files (*.*)\0*.*\0");
const TCHAR szFileExt_WAV[] = _T("wav");

const TCHAR* apszFileFilters[] = {
	szFileFilter_SMP, szFileFilter_INS, szFileFilter_MUS, szFileFilter_SFX,
	szFileFilter_ALL, NULL
};


/* BOOL MenuMan::openFile(FileType_t eMode)
 *  Attempt to select a file whose type is specified by eMode using the windows
 * file open dialog, storing its name and location in mszOpenName.
 *----------------------------------------------------------------------------*/
BOOL MenuMan::openFile(FileType_t eMode)
{
	mszOpenName[0] = _T('\0');

	if(eMode >= eFILETYPE_FOOTER)
		return false;

	mOpenParams.lpstrFilter = apszFileFilters[eMode];
	return GetOpenFileName(&mOpenParams);
}

/* BOOL MenuMan::setupLog(void)
 * Prompt the user to select a *.WAV file to save to, caching its name and
 * location in mszSaveName. Returns true if a valid target was chosen.
 *----------------------------------------------------------------------------*/
BOOL MenuMan::setupLog(void)
{
	mszSaveName[0] = _T('\0');

	return GetSaveFileName(&mSaveParams);
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
MenuMan::MenuMan(HWND hParent)
{
	mhParent = hParent;

	mhMain = GetMenu(mhParent);
	mhSub_File = GetSubMenu(mhMain, MFILE_SLOT);
	mhSub_Play = GetSubMenu(mhMain, MPLAY_SLOT);
	mhSub_Debug = GetSubMenu(mhMain, MDEBUG_SLOT);
	mhSub_Help = GetSubMenu(mhMain, MHELP_SLOT);


	/* ---- FILE OPEN PARAMETERS ----
	 */
	mOpenParams.lStructSize = sizeof(OPENFILENAME);
	mOpenParams.hwndOwner = mhParent;
	mOpenParams.hInstance = NULL;
	mOpenParams.lpstrFilter = szFileFilter_ALL;
	mOpenParams.lpstrCustomFilter = NULL;
	mOpenParams.nMaxCustFilter = 0;
	mOpenParams.nFilterIndex = 0;
	mOpenParams.lpstrFile = mszOpenName;
	mOpenParams.nMaxFile = MENUMAN_BUFFER_LEN;
	mOpenParams.lpstrFileTitle = NULL;
	mOpenParams.nMaxFileTitle = 0;
	mOpenParams.lpstrInitialDir = NULL;
	mOpenParams.lpstrTitle = NULL;
	mOpenParams.Flags = (OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY);
	mOpenParams.nFileOffset = 0;
	mOpenParams.nFileExtension = 0;
	mOpenParams.lpstrDefExt = NULL;
	mOpenParams.lCustData = 0;
	mOpenParams.lpfnHook = NULL;
	mOpenParams.lpTemplateName = NULL;


	/* ---- FILE SAVE PARAMETERS ----
	 */
	mSaveParams.lStructSize = sizeof(OPENFILENAME);
	mSaveParams.hwndOwner = mhParent;
	mSaveParams.hInstance = NULL;
	mSaveParams.lpstrFilter = szFileFilter_WAV;
	mSaveParams.lpstrCustomFilter = NULL;
	mSaveParams.nMaxCustFilter = 0;
	mSaveParams.nFilterIndex = 0;
	mSaveParams.lpstrFile = mszSaveName;
	mSaveParams.nMaxFile = MENUMAN_BUFFER_LEN;
	mSaveParams.lpstrFileTitle = NULL;
	mSaveParams.nMaxFileTitle = 0;
	mSaveParams.lpstrInitialDir = NULL;
	mSaveParams.lpstrTitle = NULL;
	mSaveParams.Flags = (OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT);
	mSaveParams.nFileOffset = 0;
	mSaveParams.nFileExtension = 0;
	mSaveParams.lpstrDefExt = szFileExt_WAV;
	mSaveParams.lCustData = 0;
	mSaveParams.lpfnHook = NULL;
	mSaveParams.lpTemplateName = NULL;

	mszBuffer[0] = _T('\0');
	mszOpenName[0] = _T('\0');
	mszSaveName[0] = _T('\0');

	lstrcpy(mszSample, szFileNone);
	lstrcpy(mszMacros, szFileNone);
	lstrcpy(mszMusic, szFileNone);
	lstrcpy(mszSFX, szFileNone);
}

MenuMan::~MenuMan(void)
{
	
}




/******************************************************************************
 * !!!!----                      ABOUT DIALOG                        ----!!!!
 ******************************************************************************/
const TCHAR szAboutDriver[] = _T(
	"Macro Channels : %u\n"
	"Render && Decode Rates : %uHz, %uHz\n"
	"Sample Library : %u Entries, %u Samples Each (Max)\n");

/* BOOL CALLBACK aboutProc(HWND hWnd, UINT uMsg,
 *                         WPARAM wParam, LPARAM lParam)
 *  Callback for the "About" dialog, which really isn't complex enough to
 * warrant its own file.
 *----------------------------------------------------------------------------*/
BOOL CALLBACK aboutProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	HWND hDriver = NULL;
	TCHAR szRender[256];

	UINT uiChannels;
	UINT uiRender,uiDecode;
	UINT uiSamples,uiLen;

	switch(uMsg)
	{
		/**
		 * ---- WM_INITDIALOG ----
		 * Received once the dialog is ready and all the controls have
		 * been created, used to set up anything akin to WM_CREATE's
		 * traditional use in normal windows.
		 */
		case WM_INITDIALOG:
			hDriver = GetDlgItem(hWnd, IDC_ABOUT_DRIVER);
			if(NULL != hDriver)
			{
				wt_getInfo(
					&uiChannels,
					&uiRender, &uiDecode,
					&uiSamples, &uiLen);

				sprintf(szRender, szAboutDriver,
					uiChannels,
					uiRender, uiDecode,
					uiSamples, uiLen);

				SetWindowText(hDriver, szRender);
			}
			break;

		/**
		 * ---- WM_CLOSE ----
		 */
		case WM_CLOSE:
			EndDialog(hWnd, IDCANCEL);
			break;

		/**
		 * ---- WM_COMMAND ----
		 * Used for button presses in this particular dialog,
		 * of which there's only one: "OK"
		 */
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDOK:
					EndDialog(hWnd, IDOK);
					break;

				case IDCANCEL:
					EndDialog(hWnd, IDCANCEL);
					break;
			}

			break;

		/**
		 * Who knows?
		 * However, we -do- know not to use DefWindowProc here,
		 * it's absolutely not for dialogs.
		 */
		default:
			return FALSE;
	}

	return TRUE;
}




/******************************************************************************
 * !!!!----                 MENU WM_COMMAND HANDLER                  ----!!!!
 ******************************************************************************/
/* LRESULT MenuMan::cmdProc(HWND hWnd, UINT uMsg,
 *                          WPARAM wParam, LPARAM lParam)
 * WM_COMMAND handler for the window process.
 *----------------------------------------------------------------------------*/
LRESULT MenuMan::cmdProc(HWND hWnd, UINT uMsg,
                         WPARAM wParam, LPARAM lParam)
{
	HRESULT hRes = E_FAIL;
	INT iStatus,iRetry;

	switch(LOWORD(wParam))
	{
		/**
		 * ---- FILE ----
		 *-----------------------------------*/
		case IDM_FILE_OPEN_SAMPLES:
			if(openFile(eFILETYPE_SMP))
			{
				lstrcpy(mszSample,
					(SUCCEEDED(pBupMan->loadSamples(mszOpenName))
						? mszOpenName : szFileNone));
			}
			break;

		case IDM_FILE_OPEN_MACROS:
			if(openFile(eFILETYPE_INS))
			{
				lstrcpy(mszMacros,
					(SUCCEEDED(pBupMan->loadMacros(mszOpenName))
						? mszOpenName : szFileNone));
			}
			break;

		case IDM_FILE_OPEN_MUSIC:
			if(openFile(eFILETYPE_MUS))
			{
				lstrcpy(mszMusic,
					(SUCCEEDED(pBupMan->loadMusic(mszOpenName))
						? mszOpenName : szFileNone));
			}
			break;

		case IDM_FILE_OPEN_SFX:
			if(openFile(eFILETYPE_SFX))
			{
				lstrcpy(mszSFX,
					(SUCCEEDED(pBupMan->loadSFX(mszOpenName))
						? mszOpenName : szFileNone));
			}
			break;

		case IDM_FILE_SAVE_WAVE:
			if(setupLog())
			{
				if(E_FAIL == pBupMan->startLog(mszSaveName))
				{
					iStatus = LoadString(GetModuleHandle(NULL),
						IDS_ERR_WAVESTART, mszBuffer, MENUMAN_BUFFER_LEN);

					if(iStatus > 0)
					{
						MessageBox(hWnd, mszBuffer, NULL,
							(MB_OK | MB_ICONEXCLAMATION));
					}
				}
			}
			break;

		case IDM_FILE_STOP_WAVE:
			pBupMan->stopLog();
			break;

		case IDM_FILE_EXIT:
			PostMessage(hWnd, WM_CLOSE, 0, 0);
			break;


		/**
		 * ---- PLAY ----
		 *-----------------------------------*/
		case IDM_PLAY_PAUSE:
			pBupMan->pause();
			break;

		case IDM_PLAY_RESUME:
			pBupMan->resume();
			break;

		case IDM_PLAY_PLAY_MUSIC:
			pBupMan->playMusic();
			break;

		case IDM_PLAY_STOP_MUSIC:
			pBupMan->stopMusic();
			break;

		case IDM_PLAY_STOP_ALL:
			pBupMan->stopAll();
			break;


		/**
		 * ---- DEBUG ----
		 *-----------------------------------*/
		case IDM_DEBUG_CHANNELS:
			if(NULL == pPatchPop)
			{
				pPatchPop = new PatchPop(hWnd);
				if(NULL == pPatchPop)
				{
					break;
				}
			}
			
			pPatchPop->createPop();
			break;

		case IDM_DEBUG_MUSIC:
			if(NULL == pMusicPop)
			{
				pMusicPop = new MusicPop(hWnd);
				if(NULL == pMusicPop)
				{
					break;
				}
			}
			
			pMusicPop->createPop();
			break;

		case IDM_DEBUG_SFX:
			if(NULL == pEffectPop)
			{
				pEffectPop = new EffectPop(hWnd);
				if(NULL == pEffectPop)
				{
					break;
				}
			}
			
			pEffectPop->createPop();
			break;


		/**
		 * ---- ABOUT ----
		 *-----------------------------------*/
		case IDM_HELP_ABOUT:
			DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ABOUT),
				hWnd, aboutProc);
			break;
	}

	/**
	 * If both the sample and instrument packages have been loaded and
	 * the driver is currently not running, try to start it.
	 */
	if(!(pBupMan->mbRunning) && pBupMan->mbGotSamples && pBupMan->mbGotMacros)
	{
		do{
			iRetry = 0;
			hRes = pBupMan->startDriver();

			if(FAILED(hRes))
			{
				iStatus = LoadString(GetModuleHandle(NULL), IDS_ERR_BUPSTART,
					mszOpenName, MENUMAN_BUFFER_LEN);

				if(iStatus > 0)
				{
					iStatus = MessageBox(hWnd, mszOpenName, NULL,
						(MB_RETRYCANCEL | MB_ICONEXCLAMATION));
					iRetry = (IDRETRY == iStatus)
						? -1 : 0;
				}
			}
		}while(iRetry);
	}

	refresh();
	DrawMenuBar(hWnd);
	return 0;
}


/******************************************************************************
 * !!!!----                         REFRESH                          ----!!!!
 ******************************************************************************/
/* void MenuMan::refresh(void)
 * Update the entire menu tree based upon the current driver state.
 *----------------------------------------------------------------------------*/
void MenuMan::refresh(void)
{
	/* ---- FILE ----
	 */
	EnableMenuItem(mhSub_File, MFILE_SUBSLOT_OPEN_MUSIC,
		(pBupMan->mbRunning
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));

	EnableMenuItem(mhSub_File, MFILE_SUBSLOT_LOG_OUTPUT,
		((pBupMan->mbRunning && !(pBupMan->mbLogging))
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_File, MFILE_SUBSLOT_STOP_LOGGING,
		((pBupMan->mbRunning && pBupMan->mbLogging)
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));


	/* ---- PLAY ----
	 */
	EnableMenuItem(mhMain, MPLAY_SLOT,
		(pBupMan->mbRunning
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));

	EnableMenuItem(mhSub_Play, MPLAY_SUBSLOT_PAUSE,
		(!(pBupMan->isPaused())
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_Play, MPLAY_SUBSLOT_RESUME,
		(pBupMan->isPaused()
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));

	EnableMenuItem(mhSub_Play, MPLAY_SUBSLOT_PLAY_MUSIC,
		(pBupMan->mbGotMusic
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_Play, MPLAY_SUBSLOT_STOP_MUSIC,
		(pBupMan->mbGotMusic
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));


	/* ---- DEBUG ----
	 */
	EnableMenuItem(mhMain, MDEBUG_SLOT,
		(pBupMan->mbRunning
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
}
