/******************************************************************************
 * panelman.cpp
 * Main button panel manager : load, play, pause, stop, record, etc.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----                      PANEL LAYOUT                        ----!!!!
 ******************************************************************************/
/* Button Boutique
 *----------------------------------------------------------------------------*/
#define PANEL_ICON_WIDTH		16
#define PANEL_ICON_HEIGHT		16

const INT aiPanelIcons[] = {
	IDI_SAMPLE_READY, IDI_SAMPLE_DONE,
	IDI_MACRO_READY, IDI_MACRO_DONE,
	IDI_OPEN_GREY, IDI_OPEN_READY, IDI_OPEN_DONE,
	IDI_RECORD_GREY, IDI_RECORD_READY, IDI_RECORD_BUSY,

	IDI_PLAY_GREY, IDI_PLAY_READY, IDI_PLAY_BUSY,
	IDI_PAUSE_GREY, IDI_PAUSE_READY, IDI_PAUSE_BUSY,
	IDI_STOP_GREY, IDI_STOP_READY,

	IDI_SPEAKER_GREY
};
#define TOTAL_PANEL_ICONS	(sizeof(aiPanelIcons) / sizeof(aiPanelIcons[0]))


const struct {
	INT iD,iCon;
	INT iGap,iWidth;

	TCHAR *szName;
	buttonCall_t pCall;
} aPanelButtons[] = {
	{PANELMAN_ID_OPEN_SAMPLE, IDI_SAMPLE_READY,
		PANELMAN_BUTTON_F_SPACING, PANELMAN_S_BUTTON_WIDTH,
		_T("Main Panel Button (OPEN SAMPLE)"), NULL},
	{PANELMAN_ID_OPEN_MACRO, IDI_MACRO_READY,
		PANELMAN_BUTTON_B_SPACING, PANELMAN_S_BUTTON_WIDTH,
		_T("Main Panel Button (OPEN MACRO)"), NULL},
	{PANELMAN_ID_OPEN_MUSIC, IDI_OPEN_GREY,
		PANELMAN_BUTTON_B_SPACING, PANELMAN_S_BUTTON_WIDTH,
		_T("Main Panel Button (OPEN MUSIC)"), NULL},
	{PANELMAN_ID_RECORD, IDI_RECORD_GREY,
		PANELMAN_BUTTON_B_SPACING, PANELMAN_S_BUTTON_WIDTH,
		_T("Main Panel Button (RECORD)"), NULL},

	{PANELMAN_ID_PLAY, IDI_PLAY_GREY,
		PANELMAN_BUTTON_G_SPACING, PANELMAN_W_BUTTON_WIDTH,
		_T("Main Panel Button (PLAY)"), NULL},
	{PANELMAN_ID_PAUSE, IDI_PAUSE_GREY,
		PANELMAN_BUTTON_B_SPACING, PANELMAN_W_BUTTON_WIDTH,
		_T("Main Panel Button (PAUSE)"), NULL},
	{PANELMAN_ID_STOP, IDI_STOP_GREY,
		PANELMAN_BUTTON_B_SPACING, PANELMAN_W_BUTTON_WIDTH,
		_T("Main Panel Button (STOP)"), NULL}
};
#define TOTAL_PANEL_BUTTONS	(sizeof(aPanelButtons) / sizeof(aPanelButtons[0]))

#define BUTTON_OPEN_SAMPLE		0
#define BUTTON_OPEN_MACRO		1
#define BUTTON_OPEN_MUSIC		2
#define BUTTON_RECORD			3

#define BUTTON_PLAY				4
#define BUTTON_PAUSE			5
#define BUTTON_STOP				6


/* Slider Salon
 *----------------------------------------------------------------------------*/
#define VOLUME_MIN				0
#define VOLUME_MAX				127

const TCHAR szSpeakerName[] = _T("Main Panel Speaker Icon");
const TCHAR szSliderName[] = _T("Main Panel Volume Trackbar");




/******************************************************************************
 * !!!!----                       REFRESH                            ----!!!!
 ******************************************************************************/
/* HICON PanelMan::getIcon(INT iRes)
 *----------------------------------------------------------------------------*/
HICON PanelMan::getIcon(INT iRes)
{
	HICON hCon = NULL;
	UINT uiCur;

	for(uiCur = 0; uiCur < TOTAL_PANEL_ICONS; uiCur++)
	{
		if(mpIcons[uiCur].iCon == iRes)
		{
			hCon = mpIcons[uiCur].hCon;
			break;
		}
	}

	return hCon;
}

/* void PanelMan::refresh(void)
 *----------------------------------------------------------------------------*/
void PanelMan::refresh(void)
{
	HICON hCon;

	/* ---- OPEN SAMPLE, MACRO, MUSIC, and START / STOP RECORDING ----
	 */
	hCon = pBupMan->mbGotSamples
		? getIcon(IDI_SAMPLE_DONE)
		: getIcon(IDI_SAMPLE_READY);
	SendMessage(mpButtons[BUTTON_OPEN_SAMPLE].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);

	hCon = pBupMan->mbGotMacros
		? getIcon(IDI_MACRO_DONE)
		: getIcon(IDI_MACRO_READY);
	SendMessage(mpButtons[BUTTON_OPEN_MACRO].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);

	hCon = pBupMan->mbRunning
		? (pBupMan->mbGotMusic
			? getIcon(IDI_OPEN_DONE)
			: getIcon(IDI_OPEN_READY))
		: getIcon(IDI_OPEN_GREY);
	SendMessage(mpButtons[BUTTON_OPEN_MUSIC].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);

	hCon = pBupMan->mbRunning
		? (pBupMan->mbLogging
			? getIcon(IDI_RECORD_BUSY)
			: getIcon(IDI_RECORD_READY))
		: getIcon(IDI_RECORD_GREY);
	SendMessage(mpButtons[BUTTON_RECORD].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);


	/* ---- PLAY, PAUSE, and STOP MUSIC ----
	 */
	hCon = pBupMan->mbRunning
		? ((0 != pBupMan->checkMusic())
			? getIcon(IDI_PLAY_BUSY)
			: getIcon(IDI_PLAY_READY))
		: getIcon(IDI_PLAY_GREY);
	SendMessage(mpButtons[BUTTON_PLAY].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);

	hCon = pBupMan->mbRunning
		? ((0 != pBupMan->isPaused())
			? getIcon(IDI_PAUSE_BUSY)
			: getIcon(IDI_PAUSE_READY))
		: getIcon(IDI_PAUSE_GREY);
	SendMessage(mpButtons[BUTTON_PAUSE].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);

	hCon = pBupMan->mbRunning
		? getIcon(IDI_STOP_READY)
		: getIcon(IDI_STOP_GREY);
	SendMessage(mpButtons[BUTTON_STOP].hWnd, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)hCon);
}




/******************************************************************************
 * !!!!----                    MESSAGE HANDLERS                      ----!!!!
 ******************************************************************************/
/* LRESULT PanelMan::cmdProc(HWND hWnd, UINT uMsg,
 *                           WPARAM wParam, LPARAM lParam)
 * From somewhere, a button...
 *----------------------------------------------------------------------------*/
LRESULT PanelMan::cmdProc(HWND hWnd, UINT uMsg,
                          WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam))
	{
		case PANELMAN_ID_OPEN_SAMPLE:
			SendMessage(hWnd, WM_COMMAND, IDM_FILE_OPEN_SAMPLES, 0);
			break;
		case PANELMAN_ID_OPEN_MACRO:
			SendMessage(hWnd, WM_COMMAND, IDM_FILE_OPEN_MACROS, 0);
			break;
		case PANELMAN_ID_OPEN_MUSIC:
			if(pBupMan->mbRunning)
			{
				SendMessage(hWnd, WM_COMMAND, IDM_FILE_OPEN_MUSIC, 0);
			}
			break;
		case PANELMAN_ID_RECORD:
			if(pBupMan->mbRunning)
			{
				SendMessage(hWnd, WM_COMMAND,
					(pBupMan->mbLogging
						? IDM_FILE_STOP_WAVE
						: IDM_FILE_SAVE_WAVE),
					0);
			}
			break;

		case PANELMAN_ID_PLAY:
			if(pBupMan->mbRunning && pBupMan->mbGotMusic)
			{
				SendMessage(hWnd, WM_COMMAND, IDM_PLAY_PLAY_MUSIC, 0);
			}
			break;
		case PANELMAN_ID_PAUSE:
			if(pBupMan->mbRunning)
			{
				SendMessage(hWnd, WM_COMMAND,
					((0 != pBupMan->isPaused())
						? IDM_PLAY_RESUME
						: IDM_PLAY_PAUSE),
					0);
			}
			break;
		case PANELMAN_ID_STOP:
			if(pBupMan->mbRunning)
			{
				SendMessage(hWnd, WM_COMMAND, IDM_PLAY_STOP_MUSIC, 0);
			}
			break;

		default:
			break;
	}

	return 0;
}

/* LRESULT PanelMan::trackProc(HWND hWnd, UINT uMsg,
 *                            WPARAM wParam, LPARAM lParam)
 * From somewhere, a trackbar...
 *----------------------------------------------------------------------------*/
LRESULT PanelMan::trackProc(HWND hWnd, UINT uMsg,
                            WPARAM wParam, LPARAM lParam)
{
	DWORD dwPos;
	BYTE cVol;

	dwPos = SendMessage(mhVolume, TBM_GETPOS, 0, 0);
	dwPos = (dwPos > VOLUME_MAX)
		? VOLUME_MAX
		: ((dwPos < VOLUME_MIN) ? VOLUME_MIN : dwPos);
	cVol = (BYTE)dwPos;

	if(pBupMan->mbRunning)
	{
		pBupMan->attenMusic(cVol);
		mdwVol = dwPos;
	}

	SendMessage(mhVolume, TBM_SETPOS, (WPARAM)TRUE, (LPARAM)mdwVol);
	return 0;
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
PanelMan::PanelMan(HWND hParent, INT iX, INT iY)
{
	UINT uiCur,uiCon;
	INT iOff;

	mhParent = hParent;
	miX = iX;
	miY = iY;

	/* ---- LOAD ICONS ----
	 */
	mpIcons = NULL;
	mpIcons = new Icon_t[TOTAL_PANEL_ICONS];
	for(uiCur = 0; uiCur < TOTAL_PANEL_ICONS; uiCur++)
	{
		mpIcons[uiCur].iCon = aiPanelIcons[uiCur];

		mpIcons[uiCur].hCon = (HICON)LoadImage(
			GetModuleHandle(NULL),
			MAKEINTRESOURCE(mpIcons[uiCur].iCon),
			IMAGE_ICON,
			PANEL_ICON_WIDTH, PANEL_ICON_HEIGHT,
			LR_DEFAULTCOLOR);
	}


	/* ---- CREATE BUTTONS ----
	 */
	mpButtons = NULL;
	mpButtons = new Button_t[TOTAL_PANEL_BUTTONS];

	iOff = iX;
	for(uiCur = 0; uiCur < TOTAL_PANEL_BUTTONS; uiCur++)
	{
		mpButtons[uiCur].pCall = aPanelButtons[uiCur].pCall;
		iOff += aPanelButtons[uiCur].iGap;

		for(uiCon = 0; uiCon < TOTAL_PANEL_ICONS; uiCon++)
		{
			if(aPanelButtons[uiCur].iCon == mpIcons[uiCon].iCon)
				break;
		}

		mpButtons[uiCur].hWnd = CreateWindowEx(
			(0),									/* dwExStyle */
			WC_BUTTON,								/* lpClassName */
			aPanelButtons[uiCur].szName,			/* lpWindowName */
			(WS_CHILD | WS_VISIBLE)
				| (BS_ICON | BS_PUSHBUTTON),		/* dwStyle */
			iOff,									/* x */
			(iY + PANELMAN_BUTTON_V_SPACING),		/* y */
			aPanelButtons[uiCur].iWidth,			/* nWidth */
			PANELMAN_BUTTON_HEIGHT,					/* nHeight */
			mhParent,								/* hWndParent */
			(HMENU)(aPanelButtons[uiCur].iD),		/* idControl */
			GetModuleHandle(NULL),					/* hInstance */
			NULL);									/* lpParam */

		if(uiCon < TOTAL_PANEL_ICONS)
		{
			SendMessage(mpButtons[uiCur].hWnd, BM_SETIMAGE,
				(WPARAM)IMAGE_ICON,
				(LPARAM)(mpIcons[uiCon].hCon));
		}

		iOff += aPanelButtons[uiCur].iWidth;
	}


	/* ---- VOLUME LABEL (SPEAKER) ----
	 */
	iOff += PANELMAN_SPEAKER_H_SPACING;
	mhSpeaker = CreateWindowEx(
		(0),										/* dwExStyle */
		WC_STATIC,									/* lpClassName */
		szSpeakerName,								/* lpWindowName */
		(WS_CHILD | WS_VISIBLE) | (SS_ICON),		/* dwStyle */
		iOff,										/* x */
		(iY + PANELMAN_SPEAKER_V_SPACING),			/* y */
		PANELMAN_SPEAKER_I_WIDTH,					/* nWidth */
		PANELMAN_SPEAKER_I_HEIGHT,					/* nHeight */
		mhParent,									/* hWndParent */
		(HMENU)PANELMAN_ID_VOL_ICON,				/* idControl */
		GetModuleHandle(NULL),						/* hInstance */
		NULL);

	for(uiCon = 0; uiCon < TOTAL_PANEL_ICONS; uiCon++)
	{
		if(IDI_SPEAKER_GREY == mpIcons[uiCon].iCon)
			break;
	}

	if(uiCon < TOTAL_PANEL_ICONS)
	{
		SendMessage(mhSpeaker, STM_SETICON,
			(WPARAM)(mpIcons[uiCon].hCon),
			(LPARAM)NULL);
	}

	iOff += PANELMAN_SPEAKER_I_WIDTH;


	/* ---- VOLUME TRACKBAR ----
	 */
	iOff += PANELMAN_VOLUME_H_SPACING;
	mhVolume = CreateWindowEx(
		(0),										/* dwExStyle */
		TRACKBAR_CLASS,								/* lpClassName */
		szSliderName,								/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (TBS_HORZ | TBS_NOTICKS),				/* dwStyle */
		iOff,										/* x */
		(iY + PANELMAN_VOLUME_V_SPACING),			/* y */
		PANELMAN_VOLUME_I_WIDTH,					/* nWidth */
		PANELMAN_VOLUME_I_HEIGHT,					/* nHeight */
		mhParent,									/* hWndParent */
		(HMENU)PANELMAN_ID_VOL_SLIDER,				/* idControl */
		GetModuleHandle(NULL),						/* hInstance */
		NULL);										/* lpParam */

	mdwVol = CORETONE_DEFAULT_VOLUME;
	SendMessage(mhVolume, TBM_SETRANGE,
		(WPARAM)TRUE,
		(LPARAM)MAKELONG(0, mdwVol));
	SendMessage(mhVolume, TBM_SETPOS,
		(WPARAM)TRUE,
		(LPARAM)mdwVol);
}

PanelMan::~PanelMan(void)
{
	UINT uiX;

	if(NULL != mpIcons)
	{
		for(uiX = 0; uiX < TOTAL_PANEL_ICONS; uiX++)
		{
			if(NULL != mpIcons[uiX].hCon)
			{
				DestroyIcon(mpIcons[uiX].hCon);
				mpIcons[uiX].hCon = NULL;
			}
		}

		delete[] mpIcons;
		mpIcons = NULL;
	}

	if(NULL != mpButtons)
	{
		delete[] mpButtons;
		mpButtons = NULL;
	}
}
