/******************************************************************************
 * patchpop.cpp
 * Patch activity dialog.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 - 2016 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdint.h>
#include <stdio.h>
#include <math.h>

#include "playtone.hpp"

/******************************************************************************
 * !!!!----                     DIALOG CALLBACK                      ----!!!!
 ******************************************************************************/
/* BOOL CALLBACK patchProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 *  Callback for the "Patch Activity" dialog.
 *----------------------------------------------------------------------------*/
BOOL CALLBACK patchProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		/**
		 * ---- WM_INITDIALOG ----
		 * Received once the dialog is ready and all the controls have
		 * been created, used to set up anything akin to WM_CREATE's
		 * traditional use in normal windows.
		 */
		case WM_INITDIALOG:
			pPatchPop->setupList(hWnd);
			break;

		/**
		 * ---- WM_CLOSE ----
		 */
		case WM_CLOSE:
			EndDialog(hWnd, IDCANCEL);
			pPatchPop->closePop(false);
			break;

		/**
		 * Who knows?
		 * However, we -do- know not to use DefWindowProc here,
		 * it's absolutely not for dialogs.
		 */
		default:
			return FALSE;
	}

	return TRUE;
}




/******************************************************************************
 * !!!!----                  LIST CONTENT CONTROL                    ----!!!!
 ******************************************************************************/
const struct {
	TCHAR *szName;
	INT fmt,width;
} aPatchLabels[] = {
	{_T("Channel"),			LVCFMT_LEFT,			52},
	{_T("Priority"),		LVCFMT_CENTER,			48},
	{_T("Type"),			LVCFMT_CENTER,			40},
	{_T("Sample Mode"),		LVCFMT_CENTER,			112},
	{_T("Phase"),			LVCFMT_CENTER,			152},
	{_T("Offset"),			LVCFMT_CENTER,			152},
	{_T("Pitch"),			LVCFMT_CENTER,			152},
	{_T("Base"),			LVCFMT_CENTER,			80},
	{_T("Pan (L)"),			LVCFMT_CENTER,			48},
	{_T("Pan (R)"),			LVCFMT_CENTER,			48},
	{_T("Pan (C)"),			LVCFMT_CENTER,			48},
	{_T("Volume"),			LVCFMT_CENTER,			104},
	{_T("Decode Offset"),	LVCFMT_CENTER,			84},
	{_T("Decode Delay"),	LVCFMT_CENTER,			80},
	{_T("Stack Level"),		LVCFMT_CENTER,			72}
};

#define PATCHPOP_COLUMN_CHANNEL			0
#define PATCHPOP_COLUMN_PRIORITY		1
#define PATCHPOP_COLUMN_TYPE			2
#define PATCHPOP_COLUMN_SAMPLE_MODE		3
#define PATCHPOP_COLUMN_PHASE			4
#define PATCHPOP_COLUMN_OFFSET			5
#define PATCHPOP_COLUMN_PITCH			6
#define PATCHPOP_COLUMN_BASE			7
#define PATCHPOP_COLUMN_PAN_L			8
#define PATCHPOP_COLUMN_PAN_R			9
#define PATCHPOP_COLUMN_PAN_C			10
#define PATCHPOP_COLUMN_VOLUME			11
#define PATCHPOP_COLUMN_DECODE_OFF		12
#define PATCHPOP_COLUMN_DECODE_DEL		13
#define PATCHPOP_COLUMN_STACK			14
#define PATCHPOP_COLUMNS				15

#define PATCHPOP_ITEM_SIZE				32


/* void PatchPop::setupList(HWND hWnd)
 *----------------------------------------------------------------------------*/
void PatchPop::setupList(HWND hWnd)
{
	LVCOLUMN lvCol;
	LVITEM lvItem;

	UINT uiC,uiR;
	TCHAR *pszCur;

	if(NULL == mhList)
	{
		mhList = GetDlgItem(hWnd, IDC_PATCH_LIST);

		/* ---- COLUMNS ----
		 */
		ZeroMemory(&lvCol, sizeof(LVCOLUMN));
		lvCol.mask = (LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM);
		lvCol.fmt = aPatchLabels[0].fmt;
		lvCol.cx = aPatchLabels[0].width;
		lvCol.pszText = (TCHAR*)(aPatchLabels[0].szName);
		lvCol.cchTextMax = (lstrlen(aPatchLabels[0].szName) + 1);
		lvCol.iSubItem = 0;

		SendMessage(mhList, LVM_INSERTCOLUMN, 0, (LPARAM)&lvCol);

		for(uiC = 1; uiC < PATCHPOP_COLUMNS; uiC++)
		{
			lvCol.fmt = aPatchLabels[uiC].fmt;
			lvCol.cx = aPatchLabels[uiC].width;
			lvCol.pszText = (TCHAR*)(aPatchLabels[uiC].szName);
			lvCol.cchTextMax = (lstrlen(aPatchLabels[uiC].szName) + 1);
			lvCol.iSubItem = uiC;

			SendMessage(mhList, LVM_INSERTCOLUMN, uiC, (LPARAM)&lvCol);
		}


		/* ---- CONTENTS ----
		 */
		pollTone();

		ZeroMemory(&lvItem, sizeof(LVITEM));
		lvItem.mask = LVIF_TEXT;
		lvItem.cchTextMax = PATCHPOP_ITEM_SIZE;

		pszCur = mpszItems;
		for(uiR = 0; uiR < muiRows; uiR++)
		{
			lvItem.pszText = pszCur;
			pszCur += PATCHPOP_ITEM_SIZE;

			lvItem.iItem = uiR;
			lvItem.iSubItem = 0;
			SendMessage(mhList, LVM_INSERTITEM, 0, (LPARAM)&lvItem);

			for(uiC = 1; uiC < PATCHPOP_COLUMNS; uiC++)
			{
				lvItem.pszText = pszCur;
				pszCur += PATCHPOP_ITEM_SIZE;

				lvItem.iSubItem = uiC;
				SendMessage(mhList, LVM_SETITEM, 0, (LPARAM)&lvItem);
			}
		}
	}
}

/* void PatchPop::pushList(void)
 *----------------------------------------------------------------------------*/
void PatchPop::pushList(void)
{
	LVITEM lvItem;

	UINT uiC,uiR;
	TCHAR *pszCur;

	if(NULL != mhList)
	{
		pollTone();

		ZeroMemory(&lvItem, sizeof(LVITEM));
		lvItem.mask = LVIF_TEXT;
		lvItem.cchTextMax = PATCHPOP_ITEM_SIZE;

		pszCur = mpszItems;
		for(uiR = 0; uiR < muiRows; uiR++)
		{
			pszCur += PATCHPOP_ITEM_SIZE;

			lvItem.iItem = uiR;
			for(uiC = 1; uiC < PATCHPOP_COLUMNS; uiC++)
			{
				lvItem.pszText = pszCur;
				pszCur += PATCHPOP_ITEM_SIZE;

				lvItem.iSubItem = uiC;
				SendMessage(mhList, LVM_SETITEM, 0, (LPARAM)&lvItem);
			}
		}
	}
}

/* void PatchPop::pollTone(void)
 *----------------------------------------------------------------------------*/
void PatchPop::pollTone(void)
{
	CoreChannel_t *pChannels;
	CorePatch_t *pPatches;

	UINT uiCur;
	TCHAR *pszCur;

	wt_getState((VOID**)(&pChannels),(VOID**)(&pPatches),
		NULL, NULL, NULL, NULL, NULL);
	pszCur = mpszItems;
	for(uiCur = 0; uiCur < muiRows; uiCur++)
	{
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* PRIORITY
		 */
		sprintf(pszCur, _T("%d"), pPatches[uiCur].iPriority);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* TYPE
		 */
		if(eCHANNEL_MODE_OFF != pChannels[uiCur].eMode)
		{
			sprintf(pszCur, _T("%s"),
				((0 != pPatches[uiCur].iInstrument) ? _T("ins") : _T("sfx")));
		}
		else
		{
			sprintf(pszCur, _T("--"));
		}
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* SAMPLE MODE
		 */
		if(eCHANNEL_MODE_SINGLESHOT == pChannels[uiCur].eMode)
		{
			sprintf(pszCur, _T("--> %u"), pChannels[uiCur].usSampleLen);
		}
		else if(eCHANNEL_MODE_LOOP == pChannels[uiCur].eMode)
		{
			sprintf(pszCur, _T("%u <-> %u"),
				pChannels[uiCur].usLoopStart, pChannels[uiCur].usLoopEnd);
		}
		else
		{
			sprintf(pszCur, _T("--"));
		}
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;


		/* PHASE += PHASE ADJUSTMENT
		 */
		sprintf(pszCur, _T("$%04X.%04X += $%04X.%04X"),
			pChannels[uiCur].phaseAcc.usPair.usHi,
			pChannels[uiCur].phaseAcc.usPair.usLo,
			pChannels[uiCur].phaseAdj.usPair.usHi,
			pChannels[uiCur].phaseAdj.usPair.usLo);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* OFFSET += OFFSET ADJUSTMENT
		 */
		sprintf(pszCur, _T("$%04X.%04X += $%04X.%04X"),
			pPatches[uiCur].freqOffset.usPair.usHi,
			pPatches[uiCur].freqOffset.usPair.usLo,
			pPatches[uiCur].offsetAdj.usPair.usHi,
			pPatches[uiCur].offsetAdj.usPair.usLo);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* PITCH += PITCH ADJUSTMENT
		 */
		sprintf(pszCur, _T("$%04X.%04X += $%04X.%04X"),
			pPatches[uiCur].freqPitch.usPair.usHi,
			pPatches[uiCur].freqPitch.usPair.usLo,
			pPatches[uiCur].pitchAdj.usPair.usHi,
			pPatches[uiCur].pitchAdj.usPair.usLo);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* BASE
		 */
		sprintf(pszCur, _T("$%04X.%04X"),
			pPatches[uiCur].freqBase.usPair.usHi,
			pPatches[uiCur].freqBase.usPair.usLo);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;


		/* PAN (L,R,C)
		 */
		sprintf(pszCur, _T("%d"), pChannels[uiCur].cVolLeft);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;
		sprintf(pszCur, _T("%d"), pChannels[uiCur].cVolRight);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;
		sprintf(pszCur, _T("%d"), pChannels[uiCur].cVolMain);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* VOLUME += VOLUME ADJ
		 */
		sprintf(pszCur, _T("$%02X.%02X += $%02X.%02X"),
			pPatches[uiCur].volCur.ucPair.ucHi,
			pPatches[uiCur].volCur.ucPair.ucLo,
			pPatches[uiCur].volAdj.ucPair.ucHi,
			pPatches[uiCur].volAdj.ucPair.ucLo);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;


		/* DECODE OFFSET,DELAY
		 */
		sprintf(pszCur, _T("%d"), pPatches[uiCur].uiOffset);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;
		sprintf(pszCur, _T("%u"), pPatches[uiCur].uiDel);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;

		/* STACK LEVEL
		 */
		sprintf(pszCur, _T("%u"), pPatches[uiCur].uiStackPos);
		pszCur = pszCur + PATCHPOP_ITEM_SIZE;
	}
}




/******************************************************************************
 * !!!!----                POPUP DIALOG MANAGEMENT                   ----!!!!
 ******************************************************************************/
/* BOOL PatchPop::createPop(void)
 *----------------------------------------------------------------------------*/
BOOL PatchPop::createPop(void)
{
	if(!IsWindow(mhPop))
	{
		mhPop = CreateDialog(GetModuleHandle(NULL),
			MAKEINTRESOURCE(IDD_PATCH), mhParent, (DLGPROC)patchProc); 

		ShowWindow(mhPop, SW_SHOW);
	}

	return IsWindow(mhPop);
}

/* void PatchPop::closePop(BOOL bForce)
 *----------------------------------------------------------------------------*/
void PatchPop::closePop(BOOL bForce)
{
	if(bForce && IsWindow(mhPop))
	{
		SendMessage(mhPop, WM_CLOSE, 0, 0);
	}

	mhList = NULL;
	mhPop = NULL;
}

/* BOOL PatchPop::checkPop(void)
 *----------------------------------------------------------------------------*/
BOOL PatchPop::checkPop(void)
{
	return IsWindow(mhPop);
}

/* HWND PatchPop::getPop(void)
 *----------------------------------------------------------------------------*/
HWND PatchPop::getPop(void)
{
	return mhPop;
}




/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
PatchPop::PatchPop(HWND hParent)
{
	UINT uiR,uiC;
	TCHAR *pszCur;

	mhParent = hParent;

	mhPop = NULL;
	mhList = NULL;
	wt_getInfo(&muiRows, NULL, NULL, NULL, NULL);

	mpszItems = new TCHAR[(muiRows * PATCHPOP_COLUMNS * PATCHPOP_ITEM_SIZE)];
	pszCur = mpszItems;
	for(uiR = 0; uiR < muiRows; uiR++)
	{
		sprintf(pszCur, _T("%u"), uiR);
		pszCur += PATCHPOP_ITEM_SIZE;

		for(uiC = 1; uiC < PATCHPOP_COLUMNS; uiC++)
		{
			lstrcpy(pszCur, _T(""));;
			pszCur += PATCHPOP_ITEM_SIZE;
		}
	}
}

PatchPop::~PatchPop(void)
{
	if(NULL != mpszItems)
	{
		delete[] mpszItems;
		mpszItems = NULL;
	}
}
